#include "Status.h"
#include "Vram.h"
#include "Sprite.h"
#include "Print.h"

constexpr byte ScoreLeft = 15;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p, bool shift)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c, shift);
        ++p;
        shift = !shift;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 0 * VramRowSize + ScoreLeft / 2, "SCORE", true);   
    PrintS(Vram + 1 * VramRowSize + ScoreLeft / 2, "STAGE", true);
    PrintByteNumber2(Vram + 1 * VramRowSize + (ScoreLeft + 10) / 2, CurrentStage + 1, true);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + 0 * VramRowSize + (ScoreLeft + 6) / 2, Score, true);
    PrintC(pVram, '0', false);
}

void PrintRemain()
{
    ptr<byte> pVram;
    byte shift;
    pVram = Vram + 3 * VramRowSize + ScoreLeft / 2;
    shift = 2;
    if (RemainCount > 1) {
        const byte[] Chars = { 0x70, 0x20, 0xe0, 0xa0, };
        byte i;
        i = RemainCount - 1;
        if (i > 2) {
            pVram = Put2S(pVram, Chars, shift);
            pVram = PrintC(pVram, ' ', false);
            pVram = PrintC(pVram, i + 0x30, true);
        }
        else {
            do {
                pVram = Put2S(pVram, Chars, shift);
                shift = (shift + 2) & 3;
                --i;
            } while (i > 0);
        }
    }
    Erase2(pVram, shift);
}

void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 2 + (ScoreLeft - 9) / 2 / 2, "GAME OVER", true);
    PresentVram();
}

void PrintTimeUp() 
{
    PrintS(Vram + VramRowSize * 2 + (ScoreLeft - 7) / 2 / 2, "TIME UP", true);
    PresentVram();
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus(); 
    {
        static const byte[] TitleBytes = {
            0x1e, 0x33, 0x60, 0x6f, 0x63, 0x33, 0x1f, 0x00, //	G
            0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x3e, 0x00, //	U
            0x63, 0x73, 0x7b, 0x7f, 0x6f, 0x67, 0x63, 0x00, //	N
            0x7e, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x00, //	T
            0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0x3e, 0x00, //	U
            0x3e, 0x63, 0x60, 0x3e, 0x03, 0x63, 0x3e, 0x00, //	S
        };
        constexpr byte TitleLength = 6;
        constexpr byte TitleLeft = (ScoreLeft - TitleLength * 2) / 2;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 0 + TitleLeft / 2;
        p = TitleBytes;
        repeat (TitleLength) {
            repeat (8) {
                *pVram = *p;
                ++p;
                pVram += VramWidth;
            }
            pVram -= VramWidth * 8 - 1;
        }
        PrintS(Vram + VramRowSize * 1 + (TitleLeft + TitleLength * 2 - 4) / 2, "MINI", false);
    }

    PrintS(Vram + VramRowSize * 2 + 0 / 2 / 2, "PUSH SPACE KEY", false);
    PrintS(Vram + VramRowSize * 3 + 0 / 2 / 2, "OR SHIFT KEY", false);
    PrintS(Vram + VramRowSize * 3 + 18 / 2, "INUFUTO 2023", false);

    PresentVram();
}


void AddScore(word pts)
{
    Score += pts;
    PrintScore();
}
