include "Vram.inc"
include "VVram.inc"

ext VVramFront_

dseg
Vram_: public Vram_
    defs VramWidth*VramHeight
Backup:
    defs VVramWidth*VVramHeight

cseg
WaitLcd:
    push ax
        do
            mov al,[es:0]
            test al,80h
        while nz | wend
    pop ax
ret

WriteLcd: public WriteLcd
    call WaitLcd
    ; or al,20h
    mov [es:0],al
    mov [es:1],ah
ret

ReadLcd: public ReadLcd
    call WaitLcd
    ; or al,20h
    mov [es:0],al
    mov ah,[es:1]
ret


; void PresentVram();
cseg
PresentVram_: public PresentVram_
    push ax | push bx | push cx | push dx | push si
        mov ax,LcdSegment | mov es,ax

        mov cl,4
        mov si,Vram_
        mov ch,0
        do
            xor ah,ah
            mov bh,2
            do
                push ax
                    mov al,22h | mov ah,ch | call WriteLcd
                    mov ax,0421h | call WriteLcd
                pop ax
                mov al,20h
                mov bl,12
                do
                    mov dh,[si] | inc si
                    xor dl,dl
                    shr dx,cl
                    or ah,dh
                    call WriteLcd
                    mov ah,dl
                    dec bl
                while nz | wend
                call WriteLcd
                xor ch,20h
                dec bh
            while nz | wend
            inc ch
            cmp ch,VramHeight
        while nz | wend
    pop si | pop dx | pop cx | pop bx | pop ax
ret


; void ClearScreen();
cseg
ClearScreen_: public ClearScreen_
    push ax | push bx | push cx
        xor ax,ax
        mov bx,Vram_
        mov cx,VramWidth*VramHeight/2
        do
            mov [bx],ax
            inc bx | inc bx
        wloop
        mov bx,Backup
        mov cx,VVramWidth*VVramHeight/2
        do
            mov [bx],ax
            inc bx | inc bx
        wloop
    pop cx | pop bx | pop ax
ret


; ptr<byte> PrintC(ptr<byte> pVram, byte c, bool right);
cseg
PrintC_: public PrintC_
    push bx | push si | push di
        mov di,ax

        mov bx,CharTable
        xor al,al
        do
            cmp dl,[bx]
            jz PrintC_break
            inc bx
            inc al
            cmp al,29
        while ne | wend
        xor al,al
        PrintC_break:
        mov ah,4 | mul ah
        add ax,AsciiPattern
        mov si,ax

        or cl,cl
        if z
            push di
                mov bl,4
                do
                    mov bh,[si] | inc si

                    mov ah,bh
                    and ah,0f0h
                    mov al,[di]
                    and al,0fh
                    or al,ah
                    mov [di],al
                    add di,VramWidth
                    
                    mov ah,bh
                    shl ah,1 | shl ah,1 | shl ah,1 | shl ah,1
                    mov al,[di]
                    and al,0fh
                    or al,ah
                    mov [di],al
                    add di,VramWidth
                    
                    dec bl
                while nz | wend
            pop ax
        else
            push di
                mov bl,4
                do
                    mov bh,[si] | inc si
                    
                    mov ah,bh
                    shr ah,1 | shr ah,1 | shr ah,1 | shr ah,1
                    mov al,[di]
                    and al,0f0h
                    or al,ah
                    mov [di],al
                    add di,VramWidth
                    
                    mov ah,bh
                    and ah,0fh
                    mov al,[di]
                    and al,0f0h
                    or al,ah
                    mov [di],al
                    add di,VramWidth
                    
                    dec bl
                while nz | wend
            pop ax
            inc ax
        endif
    pop di | pop si | pop bx
ret
CharTable:
    defb " 0123456789ACEFGHIKMNOPRSTUVY"
AsciiPattern: public AsciiPattern
	; ascii 29
	; " 0123456789ACEFGHIKMNOPRSTUVY"
	defb $00,$00,$00,$00,$ea,$aa,$e0,$00
	defb $22,$22,$20,$00,$e2,$e8,$e0,$00
	defb $e2,$e2,$e0,$00,$aa,$e2,$20,$00
	defb $e8,$e2,$e0,$00,$e8,$ea,$e0,$00
	defb $e2,$44,$40,$00,$ea,$ea,$e0,$00
	defb $ea,$e2,$e0,$00,$4a,$ae,$a0,$00
	defb $4a,$8a,$40,$00,$e8,$c8,$e0,$00
	defb $e8,$c8,$80,$00,$68,$aa,$40,$00
	defb $aa,$ea,$a0,$00,$e4,$44,$e0,$00
	defb $aa,$ca,$a0,$00,$ae,$ea,$a0,$00
	defb $ca,$aa,$a0,$00,$4a,$aa,$40,$00
	defb $ea,$e8,$80,$00,$ca,$ca,$a0,$00
	defb $68,$e2,$c0,$00,$e4,$44,$40,$00
	defb $aa,$aa,$e0,$00,$aa,$aa,$40,$00
	defb $aa,$44,$40,$00


; ptr<byte> Put(ptr<byte> pVram, byte c, byte shift);
cseg
Put: ; bx=pVram, cl=shift, dh=mask, dl=c
    mov ah,dl | and ah,0c0h | shr ah,cl
    mov al,[bx]
    and al,dh
    or al,ah
    mov [bx],al
    mov ah,dl | shl ah,1 | shl ah,1
    and ah,0c0h | shr ah,cl
    mov al,[bx+VramWidth]
    and al,dh
    or al,ah
    mov [bx+VramWidth],al
ret
Put_: public Put_
    push cx | push dx
        add cl,cl
        mov dh,03fh | ror dh,cl
        mov bx,ax
        call Put
    pop dx | pop cx
    mov ax,bx
    cmp cl,3
    if e
        inc ax
    endif
ret
        

; void VVramToVram();
VVramToVram_: public VVramToVram_
    push ax | push bx | push cx | push dx | push si | push di
        mov si,VVramFront_+VVramWidth*2
        mov di,Backup
        mov bx,Vram_
        mov ch,VVramHeight-2
        do
            push cx
                mov dh 03fh
                xor cl,cl
                mov ch,VVramWidth
                do
                    mov dl,[si] | inc si
                    cmp dl,[di]
                    if ne
                        mov [di],dl
                        call Put
                    endif
                    inc di
                    ror dh,1 | ror dh,1
                    add cl,2
                    cmp cl,8
                    if nc
                        inc bx
                        xor cl,cl
                    endif
                    dec ch
                while nz | wend
                add bx,VramWidth*2-VVramWidth/4
            pop cx
            dec ch
        while nz | wend
    pop si | pop si | pop dx | pop cx | pop bx | pop ax
ret


; ptr<byte> VramPtr(byte x, byte y);
cseg
VramPtr_: public VramPtr_
    push dx
        mov dh,al
        mov al,VramWidth*2 | mul dl
        mov dl,dh
        xor dh,dh
        add ax,dx
        add ax,Vram_
    pop dx
ret


; ptr<byte> BackupPtr(byte x, byte y);
cseg
BackupPtr_: public BackupPtr_
    push dx
        mov dh,al
        mov al,VVramWidth | mul dl
        mov dl,dh
        xor dh,dh
        add ax,dx
        add ax,Backup
    pop dx
ret


; void DrawFence();
DrawFence_: public DrawFence_
    push ax | push bx
        mov ax,2080h
        mov bx,Vram_+VVramWidth/4
        do
            mov [bx],al
            add bx,VramWidth
            xor al,80h
            dec ah
        while nz | wend
    pop bx | pop ax
ret